library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.FfpgPkg.all;

entity DelayedPulseGenerator is
    port (
        -- main signals
        Clk_ik: in std_logic;
        Reset_ir: in std_logic;
        -- trigger
        Trigger_i: in std_logic;
        -- configuration (WB)
        SetMemAddress_ob11: out unsigned(10 downto 0);
        SetMemData_ib32: in unsigned(31 downto 0);
        SetMemReadStrobe_o: out std_logic;
        ResMemAddress_ob11: out unsigned(10 downto 0);
        ResMemData_ib32: in unsigned(31 downto 0);
        ResMemReadStrobe_o: out std_logic;
        Overflow_ib16: in unsigned(15 downto 0);
        OverflowLoad_i: in std_logic;
        TriggerLatency_ib16: in unsigned(15 downto 0);
        TriggerLatencyLoad_i: in std_logic;
        Mode_i: in t_Mode;
        ModeLoad_i: in std_logic;
        Running_o: out std_logic;
        -- generation (FMC), synchronous to SerialClock_ik
        SerialClock_ik: in std_logic;
        SetStream_o: out std_logic;
        ResetStream_o: out std_logic
    );
end entity;

architecture syn of DelayedPulseGenerator is

    signal Overflow_b_slv, SerialClockOverflow_b_slv: std_logic_vector(Overflow_ib16'range);
    signal TriggerLatency_b_slv, SerialClockTriggerLatency_b_slv: std_logic_vector(TriggerLatency_ib16'range);
    signal ModeIn_slv, ModeOut_slv: std_logic_vector(1 downto 0);

    signal SerialClockReset_r: std_logic;
    signal SerialClockTrigger: std_logic;
    signal SerialClockRunning: std_logic;
    signal SerialClockOverflow_b: unsigned(Overflow_ib16'range);
    signal SerialClockTriggerLatency_b: unsigned(TriggerLatency_ib16'range);
    signal SerialClockMode: t_Mode;
    signal SerialClockModeLoad: std_logic;

begin

    ----------------------------------
    -- CDC syncers to the SerialClock_ik clock domain
    ----------------------------------

    cResetSyncer: entity work.ResetSyncer(syn)
        port map (
            Clk_ik => SerialClock_ik,
            Reset_ira => Reset_ir,
            Reset_or => SerialClockReset_r
        );

    cTriggerSyncer: entity work.Delay(syn)
        generic map (
            g_Width => 1,
            g_Delay => 3,
            g_AsyncRegUsed => "TRUE"
        )
        port map (
            Clk_ik => SerialClock_ik,
            Data_ib(0) => Trigger_i,
            Data_ob(0) => SerialClockTrigger
        );
    
    Overflow_b_slv <= std_logic_vector(Overflow_ib16);
    cOverflowSyncer: entity work.RegSyncer(syn)
        port map (
            ClkIn_ik => Clk_ik,
            ClkOut_ik => SerialClock_ik,
            Data_ib => Overflow_b_slv,
            Load_i => OverflowLoad_i,
            Data_ob => SerialClockOverflow_b_slv,
            Load_o => open
        );
    SerialClockOverflow_b <= unsigned(SerialClockOverflow_b_slv);

    TriggerLatency_b_slv <= std_logic_vector(TriggerLatency_ib16);
    cTriggerLatencySyncer: entity work.RegSyncer(syn)
        port map (
            ClkIn_ik => Clk_ik,
            ClkOut_ik => SerialClock_ik,
            Data_ib => TriggerLatency_b_slv,
            Load_i => TriggerLatencyLoad_i,
            Data_ob => SerialClockTriggerLatency_b_slv,
            Load_o => open
        );
    SerialClockTriggerLatency_b <= unsigned(SerialClockTriggerLatency_b_slv);

    ModeIn_slv <= f_ModeToSlv(Mode_i);
    cModeSyncer: entity work.RegSyncer(syn)
        port map (
            ClkIn_ik => Clk_ik,
            ClkOut_ik => SerialClock_ik,
            Data_ib => ModeIn_slv,
            Load_i => ModeLoad_i,
            Data_ob => ModeOut_slv,
            Load_o => SerialClockModeLoad
        );
    SerialClockMode <= f_SlvToMode(ModeOut_slv);

    ----------------------------------
    -- main core in SerialClock_ik clock domain
    ----------------------------------

    cRfClkDomain: entity work.RfClkDomain(syn)
        port map (
            Clk_ik => SerialClock_ik,
            Reset_ir => SerialClockReset_r,
            Overflow_ib16 => SerialClockOverflow_b,
            TriggerLatency_ib16 => SerialClockTriggerLatency_b,
            Mode_i => SerialClockMode,
            ModeLoad_i => SerialClockModeLoad,
            SetMemAddress_ob11 => SetMemAddress_ob11,
            SetMemData_ib32 => SetMemData_ib32,
            SetMemReadStrobe_o => SetMemReadStrobe_o,
            ResMemAddress_ob11 => ResMemAddress_ob11,
            ResMemData_ib32 => ResMemData_ib32,
            ResMemReadStrobe_o => ResMemReadStrobe_o,
            Running_o => SerialClockRunning,
            Trigger_i => SerialClockTrigger,
            SetStream_o => SetStream_o,
            ResetStream_o => ResetStream_o
        );

    ----------------------------------
    -- CDC syncer from the SerialClock_ik clock domain
    ----------------------------------

    cRunningSyncer: entity work.Delay(syn)
        generic map (
            g_Width => 1,
            g_Delay => 3,
            g_AsyncRegUsed => "TRUE"
        )
        port map (
            Clk_ik => Clk_ik,
            Data_ib(0) => SerialClockRunning,
            Data_ob(0) => Running_o
        );

end architecture;